import argparse

parser = argparse.ArgumentParser()
parser.add_argument(
    "project",
    nargs="?",
    metavar="PROJECT",
    help="switch current project to PROJECT, or if -e, edit PROJECT",
)
parser.add_argument("-n", "--new", metavar="PROJECT", help="create a new project")
parser.add_argument(
    "-e",
    "--edit",
    action="store_true",
    help="edit the given project, or the current project",
)
parser.add_argument("-z", "--zone", metavar="ZONE", help="specify the zone to be used")
parser.add_argument(
    "-zc", "--zone-clear", action="store_true", help="clear the zone to be used"
)
parser.add_argument(
    "--layout", action="store_true", help="specify that we want to use the layouts"
)
parser.add_argument(
    "--internalRunMode",
    required=True,
    metavar="MODE",
    help="specify the internal command that is used (e.g. project, server, etc.)",
)
parser.add_argument(
    "-l", "--list", action="store_true", help="list the available projects"
)

import os, os.path


def current_project(run_mode):
    return os.getenv("CIPLOGIC_ARCHER_CURRENT_" + run_mode.upper())


def current_zone(run_mode):
    return os.getenv("CIPLOGIC_ARCHER_CURRENT_" + run_mode.upper() + "_ZONE", "")


def archer_home(subpath):
    if "ARCHER_HOME" in os.environ:
        path = os.environ["ARCHER_HOME"]
    else:
        path = os.path.join(os.environ["HOME"], ".archer")

    path = os.path.normpath(path)

    if subpath:
        return os.path.join(path, subpath)
    else:
        return path


def project_folder(args):
    if args.layout:
        return archer_home(args.internalRunMode + "s/layouts")
    else:
        subpath = os.path.join(
            args.internalRunMode + "s", current_zone(args.internalRunMode)
        )
        return archer_home(subpath)



import os

from termcolor_util import cyan, red

from project_archer.environment.read_shell_parameters import (
    project_folder,
    current_zone,
)
from project_archer.storage.project_data import read_project_yml


def list_projects(args, env):
    folder = project_folder(args)
    zone = current_zone(args.internalRunMode)

    if zone:
        env.log(red("= " + zone, bold=True))

    items = list(sorted(os.listdir(folder)))

    available_zones = []
    for filename in items:
        if os.path.isdir(os.path.join(folder, filename)) and filename != "layouts":
            available_zones.append(filename)

    if available_zones:
        env.log("Available zones:")
        for available_zone in available_zones:
            env.log("- " + red(available_zone))

    env.log("Available projects:")
    for filename in items:
        if not os.path.isfile(os.path.join(folder, filename)):
            continue

        file_data = open(os.path.join(folder, filename))
        project_data = read_project_yml(file_data)
        env.log(
            "- "
            + cyan(os.path.splitext(filename)[0], bold=True)
            + ": "
            + project_data["name"]
        )



from project_archer.environment.environment import BashEnvironment


def clear_zone(args, env: BashEnvironment):
    env.unset_envvar(
        "CIPLOGIC_ARCHER_CURRENT_" + args.internalRunMode.upper() + "_ZONE"
    )
    env.flush()



import shlex

# Because we want to export variables to the shell running the script, we can't
# actually interact with the environment directly from the script (and have it
# persist) - instead, we output the bash commands which will modify the
# environment, which the parent shell can then |eval|


class BashEnvironment:
    def __init__(self):
        self._execution = ""

    def set_envvar(self, name, value):
        self._execution += "export " + escape(name) + "=" + shlex.quote(value) + "\n"
        return self

    def unset_envvar(self, name):
        self._execution += "unset " + escape(name) + "\n"
        return self

    def log(self, message):
        self._execution += "echo " + shlex.quote(message) + "\n"
        return self

    def define_command(self, name, body):
        self._execution += (
            escape(name)
            + "() {\n"
            + "\n\t".join(line for line in body.split("\n") if line)
            + "\n}\n"
        )
        return self

    def remove_command(self, name):
        self._execution += "unset -f " + escape(name) + "\n"
        return self

    def execute(self, body):
        self._execution += escape(body) + "\n"
        return self

    def flush(self):
        # print(self._execution.replace("\n", ";"))
        print(self._execution)
        return self


def escape(message):
    if type(message) is not str:
        message = str(message)

    return message.replace("\n", "\\n").replace("'", "\\'")



import os

from project_archer.environment.read_shell_parameters import (
    archer_home,
    current_project,
    current_zone,
)
from project_archer.storage.project_data import read_project_yml


def select_project(args, env):
    zone = current_zone(args.internalRunMode)

    if zone:
        project_name = os.path.join(zone, args.project)
    else:
        project_name = args.project

    project_data = read_project_data(project_name, args.internalRunMode)

    # 1. check if the project can be activated
    required_envvars = project_data["requires"]
    missing_envvars = [
        v
        for v in required_envvars
        if not os.getenv(v) and v not in project_data["exports"]
    ]

    if len(missing_envvars):
        env.log("Unable to activate: " + project_name + "!")
        env.log(
            "Missing environment variables: '" + "', '".join(missing_envvars) + "'."
        )
        return  # bailing out

    old_project_name = current_project(args.internalRunMode)
    if old_project_name == project_name:
        env.log(
            "The current " + args.internalRunMode + " is already: " + project_name + "."
        )
        return  # bailing out

    # 2. deactivate the previous project
    if old_project_name:
        old_project = read_project_data(old_project_name, args.internalRunMode)
        execute_commands(old_project["deactivate"], env)
        unset_commands(old_project["commands"], env)
        unset_envvars(old_project["exports"], env)

    # 3. export the environment variables
    for export_name in project_data["exports"]:
        env.set_envvar(export_name, project_data["exports"][export_name])

    # 4. activate the current project
    execute_commands(project_data["activate"], env)

    # 5. export the commands
    export_commands(project_data["commands"], env)

    # print(project_data['name'], project_name)
    env.log("Activated " + args.internalRunMode + ": " + project_data["name"])
    env.set_envvar(
        "CIPLOGIC_ARCHER_CURRENT_" + args.internalRunMode.upper(), project_name
    )


def read_project_data(project_name, internal_run_mode, projects_folder=None):
    if not projects_folder:
        projects_folder = archer_home(internal_run_mode + "s")

    project_file = os.path.join(projects_folder, project_name + ".yml")
    project_data = read_project_yml(open(project_file))

    result = {
        "name": "<none>",
        "layouts": [],
        "requires": [],
        "activate": [],
        "deactivate": [],
        "commands": {},
        "exports": {},
    }

    # this is nominally recursive - however, the |layouts| subobjects in the yml
    # files do not contain their own |layouts| entry, so this should loop should
    # not run again
    for (i, layout) in enumerate(project_data["layouts"]):
        layout_data = read_project_data(
            layout, internal_run_mode, archer_home(internal_run_mode + "s/layouts")
        )
        mix(result, layout_data)

    result["layouts"] = project_data["layouts"]
    result["name"] = project_data["name"]
    mix(result, project_data)

    return result


# mutates source
def mix(source, extra):
    source["requires"].extend(extra["requires"])
    source["deactivate"][0:0] = extra["deactivate"]
    source["activate"].extend(extra["activate"])

    for k in extra["commands"]:
        source["commands"][k] = extra["commands"][k]
    for k in extra["exports"]:
        source["exports"][k] = extra["exports"][k]


def execute_commands(commands, env):
    for command in "\n".join(commands).split("\n"):
        if command.strip():  # ignore empty commands
            env.execute(command)


def export_commands(commands, env):
    env.log("Commands: ")

    for command in commands:
        env.log("   " + command)
        env.define_command(command, commands[command])


def unset_commands(commands, env):
    for command in commands:
        env.remove_command(command)


def unset_envvars(envvars, env):
    for envvar in envvars:
        env.unset_envvar(envvar)



import sys
import io
from contextlib import redirect_stdout, redirect_stderr

import project_archer.environment.read_shell_parameters as read_shell_parameters
from project_archer.environment.environment import BashEnvironment
from project_archer.operations.clear_zone import clear_zone

from project_archer.operations.list_projects import list_projects
from project_archer.operations.create_project import create_new_project
from project_archer.operations.edit_project import edit_project
from project_archer.operations.empty_run import empty_project_run
from project_archer.operations.select_project import select_project
from project_archer.operations.select_zone import select_zone

env = BashEnvironment()

args = None
out = io.StringIO()
err = io.StringIO()
try:
    with redirect_stdout(out), redirect_stderr(err):
        args = read_shell_parameters.parser.parse_args()
except SystemExit:
    if out.getvalue():
        env.log(out.getvalue().strip())
    if err.getvalue():
        env.log(err.getvalue().strip())

if args:
    if args.list:
        list_projects(args, env)
    elif args.new:
        create_new_project(args, env)
    elif args.edit:
        edit_project(args, env)
    elif args.zone:
        select_zone(args, env)
    elif args.zone_clear:
        clear_zone(args, env)
    else:  # project selection or empty run
        if not args.project:
            empty_project_run(args, env)
        else:
            select_project(args, env)

env.flush()



from project_archer.environment.read_shell_parameters import current_project
from project_archer.operations.list_projects import list_projects


def empty_project_run(args, env):
    project = current_project(args.internalRunMode)
    if project:
        project_name = project
    else:
        project_name = "<none>"

    list_projects(args, env)
    env.log("Current " + args.internalRunMode + ": " + project_name)



import yaml


def read_project_yml(data):
    loaded_data = yaml.safe_load(data)
    try:
        project = loaded_data.get("config") or loaded_data.get("layout")
        if not project:
            project = {"name": "<broken data>"}

        ensure_array(project, "requires")
        ensure_array(project, "layouts")
        ensure_array(project, "activate")
        ensure_array(project, "deactivate")

        if "exports" not in project:
            project["exports"] = {}
        if "commands" not in project:
            project["commands"] = {}
        if "name" not in project:
            project["name"] = "<not defined>"

        return project
    except Exception as e:
        raise Exception("Unable to read loaded data: %s" % loaded_data, e)


def ensure_array(obj, property_name):
    if property_name not in obj or not obj[property_name]:
        obj[property_name] = []
    if type(obj[property_name]) is str:
        obj[property_name] = [obj[property_name]]



#!/usr/bin/env python3
import sys

from project_archer.environment.environment import BashEnvironment


def main():
    run_mode = None

    env = BashEnvironment()

    if len(sys.argv) <= 1:
        print(
            """You need to define a run mode, e.g.:

        $ . archer.py
        $ project -n test
        $ project test"""
        )
        quit(1)

    for run_mode in sys.argv[1:]:
        env.define_command(
            run_mode,
            'eval "$(%s -m project_archer.project --internalRunMode=%s $@)"'
            % ("python3", run_mode),
        )

    env.flush()


if __name__ == "__main__":
    main()



import os

from project_archer.environment.read_shell_parameters import (
    project_folder,
    current_project,
)


def edit_project(args, env):
    folder = project_folder(args)

    # if there is an argument on the shell, use that,
    # otherwise use "current project"
    if args.project:
        target_project = args.project
    else:
        target_project = current_project(args.internalRunMode)
        if not target_project:
            target_project = "undefined"
    env.execute("$EDITOR " + os.path.join(folder, target_project + ".yml"))



from project_archer.environment.environment import BashEnvironment


def select_zone(args, env: BashEnvironment):
    env.set_envvar(
        "CIPLOGIC_ARCHER_CURRENT_" + args.internalRunMode.upper() + "_ZONE", args.zone
    )
    env.flush()



import os

from project_archer.environment.read_shell_parameters import project_folder


def create_new_project(args, env):
    folder = project_folder(args)

    env.execute("mkdir -p " + folder)
    env.execute("$EDITOR " + os.path.join(folder, args.new + ".yml"))



from setuptools import setup
from setuptools import find_packages

with open("README.rst") as readme_file:
    readme = readme_file.read()

packages = find_packages()

setup(
    name="project_archer",
    version="0.3.0",
    description="Switch projects with ease.",
    long_description=readme,
    author="Bogdan Mustiata",
    author_email="bogdan.mustiata@gmail.com",
    license="BSD",
    entry_points={"console_scripts": ["archer = project_archer.mainapp:main"]},
    install_requires=[
        "termcolor-util",
        "click",
        "PyYAML >=5.1, <5.2",
    ],
    packages=packages,
    package_data={"": ["*.txt", "*.rst"]},
)
